-- Plays midi file
-- this file is public domain
-- The small sound bank is probably (c) Creative
-- ame002.mid found in libtimidity's tests (GPL2)


-- necessary for setting environment variables
local ffi = require('ffi')
ffi.cdef[[
int setenv(const char*, const char*, int);
int _putenv_s(const char*, const char*);
]]


-- prints both to screen and to console
function msg(m)
	status = (status or '') .. m .. '\n' 
	print(m)
end


-- save a single file or exit if file exists
function save_file(file_name)
	if love.filesystem.getInfo('pats/' .. file_name) then return end
	print('saving... ' .. file_name)
	love.filesystem.write('pats/' .. file_name, love.filesystem.newFileData(file_name))
end


-- used for saving many patches
function save_recursively(name)
	local files = love.filesystem.getDirectoryItems(name)
	if not files or love.filesystem.getInfo(name, 'file') then 
		save_file(name)
	else
		love.filesystem.createDirectory('pats/' .. name)
		for a = 1, #files do
			save_recursively(name .. '/' .. files[a])
		end
	end
end


function love.load()
	-- before saving or loading anything, set the identity
	-- or set it in love.conf
	love.filesystem.setIdentity('midi_test')
	
	-- copy files to 'pats' subdirectory in love's save folder
	-- should be some unique name because of checking if files exist
	love.filesystem.createDirectory('pats/')
	save_file('timidity.cfg')
	save_recursively('instruments')

	-- set path to config file as environment variable, using FFI
	-- there are some differences between Windows and Unixes
	local path = love.filesystem.getSaveDirectory() .. '/pats'
	if love.system.getOS() == 'Windows' then
		ffi.C._putenv_s('MMPAT_PATH_TO_CFG', path)
	else
		ffi.C.setenv('MMPAT_PATH_TO_CFG', path, 1)
	end

	-- print something
	msg('Using path ' .. path)
	msg('Please clean up manually')
	msg('Press Esc to quit')

	-- play song
	love.audio.newSource('ame002.mid', 'static'):play()
end


function love.draw()
	love.graphics.print(status, 10, 10)
end


function love.keypressed(k)
	if k == 'escape' then love.event.push('quit') end
end
